unit Packall;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Messages, Gauge95, TablPack, FileCtrl;

const WM_Start = WM_USER+100;

function FreeFileExt(const filename: string): string;

type
  TDlg_packall = class(TForm)
    CancelBtn: TBitBtn;
    Gauge951: TGauge95;
    L_status: TLabel;
    procedure FormShow(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
  private
    { Private declarations }
    Cancel: Boolean;
    procedure WMStart(var msg: TMessage); message WM_Start;
    procedure PackAll;
    procedure CheckAll;
  public
    { Public declarations }
    TablePack1: TTablePack;
    Files: TFileListBox;
    good, bad: integer; { Pack result }
    ckNoError, ckVerifyError, ckRepaired, ckDamaged: Integer;
  end;

var
  Dlg_packall: TDlg_packall;

implementation
uses SysUtils, Dialogs, TRepair;
{$R *.DFM}

procedure TDlg_packall.FormShow(Sender: TObject);
begin
   good := 0; bad := 0;
   ckNoError := 0; ckVerifyError := 0;
   ckRepaired := 0; ckDamaged := 0;
   Cancel := False;
   if Tag = 0 then Caption := 'Packing...'
   else Caption := 'Checking...';
   PostMessage(Handle, WM_START, 0, 0);
end;

procedure TDlg_packall.WMStart(var msg: TMessage);
begin
   with Gauge951 do begin
      MaxValue := Files.Items.Count;
      Progress := 0;
   end;
   Refresh;
   if Tag = 0 then PackAll
   else CheckAll;
end;

procedure TDlg_packall.CancelBtnClick(Sender: TObject);
begin
   Cancel := True;
end;

function FreeFileExt(const filename: string): string;
var
   szFile: array[0..128] of Char;
   i: integer;
begin
   StrPCopy(szFile, filename);
   for i:=Length(filename)-1 downto Length(filename)-4 do begin
      if szFile[i] = '.' then begin
         szFile[i] := #0;
         break;
      end;
   end;
   Result := StrPas(szFile);
end;

procedure TDlg_packall.PackAll;
var
   i: integer;
begin
   for i:=0 to Files.Items.Count-1 do
   begin
      Application.ProcessMessages;
      if Cancel then
         if MessageDlg('Cancel current packing procedure?', mtConfirmation,
                     [mbYes, mbNo], 0) = mrYes then Break;
      Cancel := False;
      Files.ItemIndex := i;
      Gauge951.Progress := i+1;
      L_status.Caption := 'Packing '+ExtractFileName(Files.FileName)
                       + ' ...';
      L_status.Refresh;
      try
         TablePack1.TableName := Files.FileName;
         try
            inc(good);
            TablePack1.Open;
            TablePack1.Pack;
         except
            L_status.Caption :=
               'Pack '+ExtractFileName(Files.FileName)+': Failed';
            L_status.Refresh;
            MessageDlg(Exception(ExceptObject).Message, mtError, [mbOk], 0);
            Inc(bad);
            Dec(good);
         end;
      finally
         TablePack1.Close;
      end;
   end;
   Close;
end;


procedure TDlg_packall.CheckAll;
var
   i: integer;
   errordir: string;
   ChkRslt: TRepairResult;
begin
   errordir := ExtractFilePath(Files.FileName); { remove end '\' }
   errordir := Copy(errordir, 1, Length(errordir)-1);
   for i:=0 to Files.Items.Count-1 do
   begin
      Application.ProcessMessages;
      if Cancel then
         if MessageDlg('Cancel current checking procedure?', mtConfirmation,
                     [mbYes, mbNo], 0) = mrYes then Break;
      Cancel := False;
      Files.ItemIndex := i;
      Gauge951.Progress := i+1;
      L_status.Caption := 'Checking '+ExtractFileName(Files.FileName)
                       + ' ...';
      L_status.Refresh;

      ChkRslt := RepairTable(FreeFileExt(Files.FileName), errordir);
      case ChkRslt of
         rrNoError: Inc(ckNoError);
         rrVerifyError: Inc(ckVerifyError);
         rrRepaired: Inc(ckRepaired);
         rrDamaged: Inc(ckDamaged);
      end;
   end;
   Close;
end;

end.
